import json
from datetime import datetime, timedelta

import pytz
from google.oauth2.credentials import Credentials
from googleapiclient.discovery import build
from loguru import logger
from quarter_lib.akeyless import get_secrets

from ..logging import setup_logging

logger = setup_logging(__file__)

SCOPES = [
    "https://www.googleapis.com/auth/calendar",
]

DEFAULT_GERMAN_OFFSET = 7200
GOOGLE_TOKEN = get_secrets("google/token.json")



def build_calendar_service():
    creds = Credentials.from_authorized_user_info(GOOGLE_TOKEN, SCOPES)
    return build("calendar", "v3", credentials=creds)

def timezone_from_datetime(dt: datetime):
    offset_in_seconds = dt.tzinfo.utcoffset(dt).seconds
    if offset_in_seconds == DEFAULT_GERMAN_OFFSET:
        return "Europe/Berlin"
    else:
        utc_offset = timedelta(seconds=offset_in_seconds)
        now = datetime.now(pytz.utc)
        return [
            tz.zone for tz in map(pytz.timezone, pytz.all_timezones_set) if now.astimezone(tz).utcoffset() == utc_offset
        ][0]

def add_event_to_calendar(summary, start: datetime, end: datetime):
    desc = {"start": start.strftime("%Y-%m-%dT%H:%M:%S%z"), "end": end.strftime("%Y-%m-%dT%H:%M:%S%z"), "app": summary}
    event = {
        "summary": summary,
        "description": json.dumps(desc, indent=4),
        "start": {
            "dateTime": start.strftime("%Y-%m-%dT%H:%M:%S"),
            "timeZone": timezone_from_datetime(start),
        },
        "end": {
            "dateTime": end.strftime("%Y-%m-%dT%H:%M:%S"),
            "timeZone": timezone_from_datetime(end),
        },
    }
    service = build_calendar_service()
    event = (
        service.events().insert(calendarId="8j088igutfd3bkcant6at54d9g@group.calendar.google.com", body=event).execute()
    )
    logger.info("Event created: %s" % (event.get("htmlLink")))

def get_dict(calendar_service):
    calendar_dict = {}
    page_token = None
    while True:
        calendar_list = calendar_service.calendarList().list(pageToken=page_token).execute()
        for calendar_list_entry in calendar_list["items"]:
            calendar_dict[calendar_list_entry["summary"]] = calendar_list_entry
        page_token = calendar_list.get("nextPageToken")
        if not page_token:
            break
    return calendar_dict

def get_events_from_calendar(calendar_name, calendar_dict, calendar_service):
    event_list = []
    page_token = None
    while True:
        events = (
            calendar_service.events()
            .list(
                calendarId=calendar_dict[calendar_name]["id"],
                pageToken=page_token,
            )
            .execute()
        )
        for event in events["items"]:
            event_list.append(event)
        page_token = events.get("nextPageToken")
        if not page_token:
            break
    return event_list



import requests

from ...logging import setup_logging

logger = setup_logging(__file__)
BASE_URL = "http://localhost:8200"


async def add_timer(summary, start, end):
    timer = {
        "summary": summary,
        "start_datetime": start,
        "end_datetime": end}
    return requests.post(BASE_URL + "/timer/add", json=timer).json()



import os

import requests
from dotenv import load_dotenv
from functools import cache

from ..logging import setup_logging

logger = setup_logging(__file__)
values_loaded = load_dotenv("cred.env")
logger.info("credentials loaded from env: " + str(values_loaded))

ACCESS_ID = os.getenv('access_id')
ACCESS_KEY = os.getenv('access_key')

AUTH_URL = "https://api.akeyless.io/auth"
SECRET_URL = "https://api.akeyless.io/get-secret-value"
TARGET_URL = "https://api.akeyless.io/get-target-details"

HEADERS = {
    "accept": "application/json",
    "content-type": "application/json"
}


@cache
def __get_token():
    token_response = requests.post(AUTH_URL, json={
        "access-type": "access_key",
        "access-id": ACCESS_ID,
        "access-key": ACCESS_KEY,
    }, headers=HEADERS).json()
    return token_response['token']


def get_secrets(secrets):
    if isinstance(secrets, str):
        secrets = [secrets]
    payload = {
        "accessibility": "regular",
        "names": secrets,
        "pretty-print": True,
        "token": __get_token()
    }
    try:
        response = requests.post(SECRET_URL, json=payload, headers=HEADERS).json()
    except Exception as e:
        logger.error(e)
        __get_token.cache_clear()
        payload = {
            "accessibility": "regular",
            "names": secrets,
            "pretty-print": True,
            "token": __get_token()
        }
        response = requests.post(SECRET_URL, json=payload, headers=HEADERS).json()
    if len(secrets) == 1:
        return response[secrets[0]]
    return [response[secret] for secret in secrets]


def get_target(name: str):
    payload = {
        "show-versions": False,
        "name": name,
        "token": __get_token()
    }
    try:
        response = requests.post(TARGET_URL, json=payload, headers=HEADERS).json()
    except Exception as e:
        logger.error(e)
        __get_token.cache_clear()
        payload = {
            "show-versions": False,
            "name": name,
            "token": __get_token()
        }
        response = requests.post(TARGET_URL, json=payload, headers=HEADERS).json()
    return list(response['value']['db_target_details'].values())



import json
import os
import subprocess
import sys
from datetime import datetime, timedelta

import speech_recognition as sr

from ..logging import setup_logging

logger = setup_logging(__file__)

def convert_to_wav(file_path):
    new_file_path = file_path + ".wav"
    wav_cmd = 'ffmpeg -y -i "' + file_path + '" -ar 48000 "' + new_file_path + '"'
    subprocess.call(wav_cmd, shell=True)
    return new_file_path

def get_recognized_text(file_path):
    r = sr.Recognizer()
    wav_converted_file_path = convert_to_wav(file_path)
    r_file = sr.AudioFile(wav_converted_file_path)
    with r_file as source:
        audio = r.record(source)
    try:
        return r.recognize_google(audio, language="de-DE"), wav_converted_file_path
    except sr.UnknownValueError as e:
        return None, wav_converted_file_path



import pymysql.cursors

from ...akeyless import get_target
from ...logging import setup_logging

logger = setup_logging(__file__)

DB_USER_NAME, DB_HOST_NAME, DB_PASSWORD, DB_PORT, DB_NAME = get_target("private")
DB_USER_NAME_MONICA, DB_HOST_NAME, DB_PASSWORD_MONICA, DB_PORT_MONICA, DB_NAME_MONICA = get_target("monica")

def create_server_connection():
    return pymysql.connect(
        host=DB_HOST_NAME,
        port=3306,
        user=DB_USER_NAME,
        password=DB_PASSWORD,
        database=DB_NAME,
        cursorclass=pymysql.cursors.DictCursor,
    )

def create_monica_server_connection():
    return pymysql.connect(
        host=DB_HOST_NAME,
        port=3306,
        user=DB_USER_NAME_MONICA,
        password=DB_PASSWORD_MONICA,
        database=DB_NAME_MONICA,
        cursorclass=pymysql.cursors.DictCursor,
    )

def close_server_connection(connection):
    connection.close()



import os

from loguru import logger


def setup_logging(file_path: str):
    logger.add(
        os.path.join(os.path.dirname(os.path.abspath(file_path)) + "/logs/" + os.path.basename(file_path) + ".log"),
        format="{time:YYYY-MM-DD at HH:mm:ss} | {level} | {message}",
        backtrace=True,
        diagnose=True,
    )
    return logger



"""
Quarter Lib - A comprehensive library for various productivity integrations.

This package provides modules for:
- Todoist integration
- Notion API
- Google Calendar integration  
- Monica CRM integration
- Voice recording and transcription
- File utilities
- Database connections (MySQL and HTTP-based)
- Akeyless secret management
- Logging utilities
"""

__version__ = "2.0.0"

# Import main modules for convenience
from . import akeyless
from . import database
from . import file_helper
from . import google_calendar
from . import logging
from . import models
from . import monica
from . import notion
from . import todoist
from . import transcriber
from . import voice_recorder

__all__ = [
    "akeyless",
    "database", 
    "file_helper",
    "google_calendar",
    "logging",
    "models",
    "monica",
    "notion",
    "todoist",
    "transcriber",
    "voice_recorder",
]



import requests

BASE_URL = "http://localhost:8100"


async def get_events(days):
    if days:
        return requests.get(BASE_URL + "/events/{days}").json()
    else:
        return requests.get(BASE_URL + "/events").json()


async def add_event(summary, start, end):
    event = {
        "summary": summary,
        "start_datetime": start,
        "end_datetime": end}
    return requests.post(BASE_URL + "/event", json=event).json()



import json
import uuid
from datetime import timedelta

import pandas as pd
import requests

from ..akeyless import get_secrets
from ..logging import setup_logging

logger = setup_logging(__file__)

NOTION_TOKEN = get_secrets("notion/token")

BASE_URL = "https://api.notion.com/v1/"
HEADERS = {
    "Authorization": "Bearer " + NOTION_TOKEN,
    "Content-Type": "application/json",
    "Notion-Version": "2021-08-16",
}

def get_databases(databases):
    result_list = []
    for database_id in databases:
        url = BASE_URL + "databases/" + database_id + "/query"
        body = None
        while True:
            r = (
                requests.post(url, headers=HEADERS).json()
                if body is None
                else requests.post(url, data=json.dumps(body), headers=HEADERS).json()
            )
            for results in r["results"]:
                result_list.append(results)
            body = {"start_cursor": r.get("next_cursor")}
            if not r["has_more"]:
                break
    return pd.json_normalize(result_list, sep="~")

def get_database(database_id):
    url = BASE_URL + "databases/" + database_id + "/query"
    result_list = []
    body = None
    while True:
        r = (
            requests.post(url, headers=HEADERS).json()
            if body is None
            else requests.post(url, data=json.dumps(body), headers=HEADERS).json()
        )
        for results in r["results"]:
            result_list.append(results)
        body = {"start_cursor": r.get("next_cursor")}
        if not r["has_more"]:
            break
    return pd.json_normalize(result_list, sep="~")



from datetime import datetime
from typing import Optional

from pydantic import BaseModel


class ZoteroTask(BaseModel):
    title: str
    author: str
    ms_of_bookmark_timestamp: int
    ms_of_bookmark_chapter: int
    timestamp: datetime


class AudiobookFinished(BaseModel):
    title: str


class DefaultModel(BaseModel):
    start: datetime
    end: datetime


class Exercise(DefaultModel):
    type: str


class MeditationSession(Exercise):
    pass


class YogaSession(Exercise):
    pass


class ReadingSession(DefaultModel):
    title: str
    page_old: int
    page_new: int
    reading_type: str
    finished: Optional[bool] = False


class NewBook(BaseModel):
    title: str
    type: str


class Timer(DefaultModel):
    context: str


class AppUsage(DefaultModel):
    app: str


class Power(BaseModel):
    type: str


class DrugSession(BaseModel):
    beer: Optional[str]
    wine: Optional[str]
    liquor: Optional[str]
    other: Optional[str]
    water: Optional[str]



from datetime import datetime
from uuid import uuid4

import pymysql.cursors

from ..database.mysql import (
    close_server_connection,
    create_monica_server_connection,
)
from ..todoist import get_default_offset
from ..logging import setup_logging

logger = setup_logging(__file__)

DEFAULT_ACCOUNT_ID = 1
INBOX_CONTACT_ID = 52
MICROJOURNAL_CONTACT_ID = 58

def add_to_journal(recognized_text, date, context="Meditation"):
    _, delta_str = get_default_offset()
    connection = create_monica_server_connection()
    try:
        with connection.cursor() as cursor:
            title = "{timestamp} - Im Kontext: {context}".format(timestamp=date.strftime("%H:%M:%S") + delta_str, context=context)
            timestamp = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
            happened_at = date.strftime("%Y-%m-%d")
            activities_values = tuple(
                (
                    uuid4(),
                    DEFAULT_ACCOUNT_ID,
                    title,
                    recognized_text,
                    happened_at,
                    timestamp,
                    timestamp,
                )
            )
            cursor.execute(
                "INSERT INTO activities (uuid, account_id, summary, description, happened_at, created_at, updated_at) VALUES (%s, %s, %s, %s, %s, %s, %s)",
                activities_values,
            )
            connection.commit()
            last_row_id = cursor.lastrowid

            activity_contact_values = tuple((last_row_id, MICROJOURNAL_CONTACT_ID, DEFAULT_ACCOUNT_ID))
            cursor.execute(
                "INSERT INTO activity_contact (activity_id, contact_id, account_id) VALUES (%s, %s, %s)",
                activity_contact_values,
            )
            connection.commit()
    except pymysql.err.IntegrityError as e:
        logger.error("IntegrityError: {error}".format(error=e))
    close_server_connection(connection)



import json
import os

from ..logging import setup_logging

logger = setup_logging(__file__)

def delete_files(files):
    for file in files:
        try:
            os.remove(file)
        except OSError as e:
            logger.error(e)

def get_config(file_path):
    with open(os.path.join(os.getcwd() + "/config/", file_path), encoding="utf-8") as f:
        data = json.load(f)
    return data

def get_value(value, row, config):
    return next(i for i in config if i[row] == value)



from datetime import datetime

from ..file_helper import delete_files
from ..monica import add_to_journal
from ..todoist import add_note_with_attachement, add_to_todoist
from ..transcriber import get_recognized_text
from ..logging import setup_logging

logger = setup_logging(__file__)

def get_meditation_logs(file_path, file_name):
    print(file_path)
    date = datetime.strptime(file_name[:-4], "%Y-%m-%d-%H-%M-%S")
    recognized_text, wav = get_recognized_text(file_path)
    if recognized_text:
        add_to_journal(recognized_text, date)
        item = add_to_todoist(
            "'{recognized_text}' was recorded during meditation at '{date}'. Anything to review?".format(
                date=date.strftime("%Y-%m-%d %H:%M:%S"), recognized_text=recognized_text
            )
        )
        add_note_with_attachement(item.id, file_path)
        delete_files([file_path, wav])
        return "Recording from '{date}' with content '{text}' was added to journal and Todoist.".format(
            date=date, text=recognized_text
        )
    else:
        delete_files([file_path, wav])
        return "Recording from {date} was not recognized and therefore not added to journal or Todoist.".format(
            date=date
        )

def get_logs_from_recording(file_path, file_name, context):
    date = datetime.strptime(file_name[:-4], "%Y-%m-%d-%H-%M-%S")
    recognized_text, wav = get_recognized_text(file_path)
    if recognized_text:
        logger.info("Recognized text '{recognized_text}' for context '{context}' at '{date}'",
                    recognized_text=recognized_text,
                    context=context, date=date)
        add_to_journal(recognized_text, date, context)
        item = add_to_todoist(
            "'{recognized_text}' was recorded with the context '{context}' at '{date}'. Anything to review?".format(
                context=context,
                date=date.strftime("%Y-%m-%d %H:%M:%S"), recognized_text=recognized_text
            )
        )
        add_note_with_attachement(item.id, file_path)
        logger.info(
            "Recording from '{date}' with content '{text}' and context '{context}' was added to Micro-Journal and Todoist.".format(
                context=context, date=date, text=recognized_text))
    else:
        logger.info("No recognized text for context '{context}' at '{date}'", context=context, date=date)
    delete_files([file_path, wav])



import uuid
from datetime import timedelta, time, datetime
from typing import Dict
from urllib.parse import urljoin

import pandas as pd
import requests
from dateutil import parser
from todoist_api_python.api import TodoistAPI

from ..akeyless import get_secrets
from ..logging import setup_logging

logger = setup_logging(__file__)

TODOIST_TOKEN = get_secrets("todoist/token")

END_TIME = time(hour=6, minute=0, second=0)
TODOIST_API = TodoistAPI(TODOIST_TOKEN)

CONTENT_TYPE = ("Content-Type", "application/json; charset=utf-8")
AUTHORIZATION = ("Authorization", "Bearer %s")
X_REQUEST_ID = ("X-Request-Id", "%s")

BASE_URL = "https://api.todoist.com"
AUTH_BASE_URL = "https://todoist.com"
SYNC_VERSION = "v9"
REST_VERSION = "v2"

SYNC_API = urljoin(BASE_URL, f"/sync/{SYNC_VERSION}/")
REST_API = urljoin(BASE_URL, f"/rest/{REST_VERSION}/")

TASKS_ENDPOINT = "tasks"
PROJECTS_ENDPOINT = "projects"
COLLABORATORS_ENDPOINT = "collaborators"
SECTIONS_ENDPOINT = "sections"
COMMENTS_ENDPOINT = "comments"
LABELS_ENDPOINT = "labels"
SHARED_LABELS_ENDPOINT = "labels/shared"
SHARED_LABELS_RENAME_ENDPOINT = f"{SHARED_LABELS_ENDPOINT}/rename"
SHARED_LABELS_REMOVE_ENDPOINT = f"{SHARED_LABELS_ENDPOINT}/remove"
QUICK_ADD_ENDPOINT = "quick/add"

AUTHORIZE_ENDPOINT = "oauth/authorize"
TOKEN_ENDPOINT = "oauth/access_token"
REVOKE_TOKEN_ENDPOINT = "access_tokens/revoke"

def get_rest_url(relative_path: str) -> str:
    return urljoin(REST_API, relative_path)

def get_sync_url(relative_path: str) -> str:
    return urljoin(SYNC_API, relative_path)

def get_auth_url(relative_path: str) -> str:
    return urljoin(AUTH_BASE_URL, relative_path)

def create_headers(
    token: str | None = None,
    with_content: bool = False,
    request_id: str | None = None,
) -> Dict[str, str]:
    headers: Dict[str, str] = {}

    if token:
        headers.update([(AUTHORIZATION[0], AUTHORIZATION[1] % token)])
    if with_content:
        headers.update([CONTENT_TYPE])
    if request_id:
        headers.update([(X_REQUEST_ID[0], X_REQUEST_ID[1] % request_id)])

    return headers

HEADERS = create_headers(TODOIST_TOKEN)

def get_activity(**kwargs):
    return requests.post(get_sync_url("activity/get"), headers=HEADERS, data=kwargs).json()

def run_sync_commands(commands):
    return requests.post(
        get_sync_url("sync"),
        headers=HEADERS,
        json={"commands": commands},
    )

def get_user_state():
    return requests.post(
        get_sync_url("sync"), headers=HEADERS, json={"sync_token": "*", "resource_types": ["user"]}
    ).json()["user"]["tz_info"]

def get_user_karma_vacation():
    return requests.post(
        get_sync_url("sync"), headers=HEADERS, json={"sync_token": "*", "resource_types": ["user"]}
    ).json()["user"]["features"]["karma_vacation"]

def upload_file(file_path):
    return requests.post(
        get_sync_url("uploads/add"),
        headers=HEADERS,
        files={
            "file": open(file_path, "rb"),
        },
    ).json()

def add_note_with_attachement(task_id, file_path):
    file = upload_file(file_path)
    return requests.post(
        get_sync_url("sync"),
        headers=HEADERS,
        json={
            "commands": [
                {
                    "type": "note_add",
                    "temp_id": str(uuid.uuid4()),
                    "uuid": str(uuid.uuid4()),
                    "args": {
                        "item_id": task_id,
                        "content": "",
                        "file_attachment": {
                            "file_name": file["file_name"],
                            "file_size": file["file_size"],
                            "file_type": file["file_type"],
                            "file_url": file["file_url"],
                            "resource_type": "audio",
                        },
                    },
                },
            ]
        },
    ).json()

def add_note_with_attachement_and_content(task_id, content, file_path):
    """Enhanced version from todoist_notes module"""
    file = upload_file(file_path)
    return requests.post(
        get_sync_url("sync"),
        headers=HEADERS,
        json={
            "commands": [
                {
                    "type": "note_add",
                    "temp_id": str(uuid.uuid4()),
                    "uuid": str(uuid.uuid4()),
                    "args": {
                        "item_id": task_id,
                        "content": content,
                        "file_attachment": {"file_url": file["file_url"]},
                    },
                }
            ]
        },
    ).json()

def add_task_to_todoist(task, project_id, description=None):
    """Enhanced task creation from todoist_notes module"""
    command = {
        "type": "item_add",
        "temp_id": str(uuid.uuid4()),
        "uuid": str(uuid.uuid4()),
        "args": {
            "content": task,
            "project_id": project_id,
        },
    }
    if description:
        command["args"]["description"] = description
    return requests.post(
        get_sync_url("sync"),
        headers=HEADERS,
        json={"commands": [command]},
    ).json()

def update_due(task_id, due, add_reminder=False):
    commands = [
        {
            "type": "item_update",
            "uuid": str(uuid.uuid4()),
            "args": {
                "id": task_id,
                "due": due,
            },
        }
    ]
    if add_reminder:
        commands.append(
            {
                "type": "reminder_add",
                "temp_id": str(uuid.uuid4()),
                "uuid": str(uuid.uuid4()),
                "args": {
                    "item_id": task_id,
                    "due": due,
                },
            }
        )
    return requests.post(
        get_sync_url("sync"),
        headers=HEADERS,
        json={"commands": commands},
    ).json()

def add_reminder(task_id, due):
    return requests.post(
        get_sync_url("sync"),
        headers=HEADERS,
        json={
            "commands": [
                {
                    "type": "reminder_add",
                    "temp_id": str(uuid.uuid4()),
                    "uuid": str(uuid.uuid4()),
                    "args": {
                        "item_id": task_id,
                        "due": due,
                    },
                }
            ]
        },
    ).json()

def move_item_to_project(task_id, project_id):
    return requests.post(
        get_sync_url("sync"),
        headers=HEADERS,
        json={
            "commands": [
                {
                    "type": "item_move",
                    "uuid": str(uuid.uuid4()),
                    "args": {
                        "id": task_id,
                        "project_id": project_id,
                    },
                }
            ]
        },
    ).json()

def move_item_to_section(task_id, section_id):
    return requests.post(
        get_sync_url("sync"),
        headers=HEADERS,
        json={
            "commands": [
                {
                    "type": "item_move",
                    "uuid": str(uuid.uuid4()),
                    "args": {
                        "id": task_id,
                        "section_id": section_id,
                    },
                }
            ]
        },
    ).json()

def add_to_todoist(text, project_id=None):
    item = TODOIST_API.add_task(text)
    if project_id:
        move_item_to_project(item.id, project_id)
    return item

def get_default_offset():
    tz_info = get_user_state()
    delta = timedelta(hours=tz_info["hours"], minutes=tz_info["minutes"])
    return delta, tz_info["gmt_string"]

def get_items_by_label(label_id):
    label = TODOIST_API.get_label(label_id)
    return TODOIST_API.get_tasks(label=label.name)

def get_items_by_project(project_id):
    items = TODOIST_API.get_tasks(project_id=project_id)
    return [item for item in items if item.is_completed == False]

async def complete_task_by_title(selected_service):
    df_items = pd.DataFrame([item.__dict__ for item in TODOIST_API.get_tasks()])
    item = df_items[df_items.content == selected_service].sample(1).iloc[0]
    logger.info(item)

    TODOIST_API.close_task(str(item["id"]))
    api_item = TODOIST_API.get_task(str(item["id"]))
    if parser.parse(api_item.due.date).date() >= (datetime.today() + timedelta(days=1)).date():
        due = {
            "date": (datetime.today() + timedelta(days=1)).strftime("%Y-%m-%d"),
            "is_recurring": True,
            "lang": "en",
            "string": "every day",
            "timezone": None,
        }
        update_due(item["id"], due=due)

    if END_TIME > datetime.now().time():
        due = {
            "date": datetime.today().strftime("%Y-%m-%d"),
            "is_recurring": True,
            "lang": "en",
            "string": "every day",
            "timezone": None,
        }
        update_due(item["id"], due=due)
